library fancy_syntax.mirrors;

import 'dart:mirrors';

/**
 * Walks up the class hierarchy to find a method declaration with
 * the given [name].
 *
 * Note that it's not possible to tell if there's an implementation
 * due to noSuchMethod().
 */
Mirror getMemberMirror(ClassMirror classMirror, Symbol name) {
  if (classMirror.members.containsKey(name)) {
    return classMirror.members[name];
  }
  if (hasSuperclass(classMirror)) {
    var mirror = getMemberMirror(classMirror.superclass, name);
    if (mirror != null) {
      return mirror;
    }
  }
  for (ClassMirror supe in classMirror.superinterfaces) {
    var mirror = getMemberMirror(supe, name);
    if (mirror != null) {
      return mirror;
    }
  }
  return null;
}

/**
 * Work-around for http://dartbug.com/5794
 */
bool hasSuperclass(ClassMirror classMirror) {
  ClassMirror superclass = classMirror.superclass;
  return (superclass != null)
      && (superclass.qualifiedName != new Symbol("dart.core.Object"));
}